/**
* Copyright (C) 2009 eXo Platform SAS.
*
* This is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as
* published by the Free Software Foundation; either version 2.1 of
* the License, or (at your option) any later version.
*
* This software is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this software; if not, write to the Free
* Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
* 02110-1301 USA, or see the FSF site: http://www.fsf.org.
*/
package com.tsky.selfcare.util.sessionlistener;

import java.security.Principal;

import javax.faces.context.FacesContext;
import javax.security.auth.login.LoginException;
import javax.servlet.http.HttpServletRequest;

import org.exoplatform.container.component.ComponentRequestLifecycle;
import org.exoplatform.container.component.RequestLifeCycle;
import org.exoplatform.portal.application.PortalRequestContext;
import org.exoplatform.services.log.ExoLogger;
import org.exoplatform.services.log.Log;
import org.exoplatform.services.organization.Group;
import org.exoplatform.services.organization.MembershipType;
import org.exoplatform.services.organization.OrganizationService;
import org.exoplatform.services.organization.User;
import org.exoplatform.services.security.Authenticator;
import org.exoplatform.services.security.Identity;
import org.exoplatform.services.security.MembershipEntry;
import org.exoplatform.services.security.jaas.AbstractLoginModule;

/**
 * Login module can be used to add authenticated user to some group after successful login.<br>
 * For example, user can be add as "member" to group "/platform/users" after his login. Group name and Membership
 * type are configurable and if they are not provided by configuration, then value "member" is used as default
 * value for membership type and "/platform/users" for group.
 *
 * @author <a href="mailto:mposolda@redhat.com">Marek Posolda</a>
 * @author <a href="mailto:vrockai@redhat.com">Viliam Rockai</a>
 * @author <a href="mailto:boleslaw.dawidowicz at redhat.com">Boleslaw Dawidowicz</a>
 */
public class SelfcareIdentityLoginModule extends AbstractLoginModule
{
   /** Logger. */

	{
		System.out.println("Control is inside SelfcareIdentityLoginModule");
	}
   private static final Log log = ExoLogger.getLogger(SelfcareIdentityLoginModule.class);

   private static final String OPTION_MEMBERSHIP_TYPE = "membershipType";

   private static final String OPTION_GROUP_ID = "groupId";

   // values obtained from configuration options
   private String membershipType;

   private String groupId;

   // MembershipEntry with values provided from configuration. We will use it to check if user is not already presented in our group.
   private MembershipEntry requestedMembershipEntry;

   /**
    * Read values from configuration. Default values ("member" and "/platform/users") are used if options are missing in configuration.
    */
   protected void afterInitialize()
   {
      membershipType = options.get(OPTION_MEMBERSHIP_TYPE) != null ? (String)options.get(OPTION_MEMBERSHIP_TYPE) : "member";
      groupId = options.get(OPTION_GROUP_ID) != null ? (String)options.get(OPTION_GROUP_ID) : "/platform/users";
      // membershipType is * so we are not checking exact value of membershipType in method login
      requestedMembershipEntry = new MembershipEntry(groupId);
   }
   /**
    * @see javax.security.auth.spi.LoginModule#login()
    */
   @SuppressWarnings("unchecked")
   public boolean login() throws LoginException
   {
	  System.out.println("Control is inside login module");
      if (log.isDebugEnabled())
      {
         log.debug("login invoked!");
      }
      try
      {
         // get identity set by SharedStateLoginModule in case of successful authentication
         Identity identity = null;
//         FacesContext context = FacesContext.getCurrentInstance();
//         HttpServletRequest request = (HttpServletRequest) context.getExternalContext().getRequest();
//         Principal userPrincipal = request.getUserPrincipal();
//         System.out.println("User Name:::::"+userPrincipal.getName());
         System.out.println("Shared State Values: " + sharedState.keySet());
         identity = (Identity)sharedState.get("exo.security.identity");
         System.out.println("Value of identity is before the IF loop is :: " +identity);
         if (sharedState.containsKey("exo.security.identity"))
         {
            identity = (Identity)sharedState.get("exo.security.identity");
            System.out.println("Value of identity is :: " +identity);
         }
         // return if identity is not present (this means that user authentication failed in SharedStateLoginModule)
         if (identity == null)
         {
        	log.info("Didnt get identity");
            return true;
         }
         // check if user is already added to our group with given membershipType. If yes, we don't need to do something.
         if (identity.getMemberships().contains(requestedMembershipEntry))
         {
        	 log.info("I am in check if user is already added to our group with given membershipType");
            return true;
         }
         log.info("User " + identity.getUserId() + " will be added to group " + groupId + " as " + membershipType + ".");
         // Now add our user to requested group
         addUserToPlatformUsers(identity.getUserId());
         // recreate identity
         Authenticator authenticator = (Authenticator)getContainer().getComponentInstanceOfType(Authenticator.class);
         identity = authenticator.createIdentity(identity.getUserId());
         sharedState.put("exo.security.identity", identity);
         return true;
      }
      catch (Exception e)
      {
         LoginException le = new LoginException();
         le.initCause(e);
         throw le;
      }
   }
   /**
    * @see javax.security.auth.spi.LoginModule#commit()
    */
   public boolean commit() throws LoginException
   {
	 System.out.println("I am in commit method of Login Module");
//	 PortalRequestContext context = PortalRequestContext.getCurrentInstance() ;
//		String remoteUser = context.getRemoteUser();
//     System.out.println("User Name:::::"+remoteUser);
	 Intialize();
      return true;
   }
   /**
    * @see javax.security.auth.spi.LoginModule#abort()
    */
   public boolean abort() throws LoginException
   {
	   System.out.println("I am in abort method of Login Module");
      return true;
   }
   /**
    * @see javax.security.auth.spi.LoginModule#logout()
    */
   public boolean logout() throws LoginException
   {
	   System.out.println("I am in logout method of Login Module");
      return true;
   }
   @Override
   protected Log getLogger()
   {
      return log;
   }
   /**
    * Add given user to our group with given membershipType.
    *
    * @param userId
    */
   private void addUserToPlatformUsers(String userId) throws Exception
   {
      try
      {
         OrganizationService orgService = (OrganizationService)getContainer().getComponentInstanceOfType(OrganizationService.class);
         begin(orgService);
         User user = orgService.getUserHandler().findUserByName(userId);
         MembershipType memberType = orgService.getMembershipTypeHandler().findMembershipType(membershipType);
         Group platformUsersGroup = orgService.getGroupHandler().findGroupById(groupId);
         orgService.getMembershipHandler().linkMembership(user, platformUsersGroup, memberType, true);
         end(orgService);
      }
      catch (Exception e)
      {
         log.error("Failed to add user " + userId + " to group " + groupId + ".", e);
         // don't rethrow login exception in case of failure.
         // throw e;
      }
   }
   private void begin(OrganizationService orgService) throws Exception
   {
      if (orgService instanceof ComponentRequestLifecycle)
      {
         RequestLifeCycle.begin((ComponentRequestLifecycle)orgService);
      }
   }
   private void end(OrganizationService orgService) throws Exception
   {
      if (orgService instanceof ComponentRequestLifecycle)
      {
         RequestLifeCycle.end();
      }
   }
   public void Intialize()
   {
	   String s="Welcome";
	   System.out.println("MEssage:::::::::::::"+s);
   }
}
